unit SerialUnit;
//{$MODE Delphi}
interface

uses
  SysUtils, {WinTypes, WinProcs,} Classes, Forms, Messages, Dialogs, ExtCtrls, Graphics
{$IFDEF Unix}
, baseunix
{$ENDIF};

  procedure        TalkToHardware(LongCount: integer; Command: byte);
  procedure        GetHardwareVersion;
  procedure        FindAnyHardware;
  function         HardwareFound: boolean;
  procedure        ResetHardware;
  procedure        OpenComm;
  procedure        TLong(x: cardinal);
  procedure        TByte(x: byte);
  procedure        TComm;
  function         RBit(Echo: boolean; Timeout: integer): byte;
  procedure        RComm;
  procedure        CloseComm;
  procedure        HardwareLostError;
  procedure        CommError(Msg: string);
  function         CommString: string;
  procedure        DelayMilliseconds(ms: cardinal);
  function         IterateLFSR: byte;
  function GetTickCount: TClock;

const
  TxBuffSize         = 256;
  RxBuffSize         = 256;

var
  CommPort           : integer;
  CommOpen           : boolean;
  CommHandle         : THandle;


  TxBuffLength       : cardinal;
  TxBuff             : array[0..TxBuffSize-1] of byte;
  RxBuffStart        : cardinal;
  RxBuffEnd          : cardinal;
  RxBuff             : array[0..RxBuffSize-1] of byte;

  LFSR               : byte;

  Version            : byte;
  VersionMode        : boolean;
  AbortMode          : boolean;

  LongCount          : integer;

implementation

uses {GlobalUnit,} Serial, crt, ProgressUnit;

/////////////////////
//  Chip Routines  //
/////////////////////

// Talk to hardware
procedure TalkToHardware(LongCount: integer; Command: byte);
var
  i: integer;
begin
  // find hardware
  VersionMode := False;
  FindAnyHardware;
     //check version here
  // send command
  TLong(Command);
  TComm;
  // if command 1-3, do the following
  if Command > 0 then
  begin
    // update progress form
    ProgressForm.StatusLabel.Caption := 'Loading RAM';
    ProgressForm.StatusLabel.Repaint;
    // send count and longs
    TLong(LongCount);
//    for i := 0 to LongCount-1 do TLong(PIntegerArray(@MemoryBuffer)[i]);
    TComm;
    // update progress form
    ProgressForm.StatusLabel.Caption := 'Verifying RAM';
    ProgressForm.StatusLabel.Repaint;
    // receive ram checksum pass/fail
    if RBit(True, 2500) = 1 then CommError('RAM checksum error on');
    // if command 2-3, do the following
    if Command > 1 then
    begin
      // update progress form
      ProgressForm.StatusLabel.Caption := 'Programming EEPROM';
      ProgressForm.StatusLabel.Repaint;
      // receive eeprom program pass/fail
      if RBit(True, 5000) = 1 then CommError('EEPROM programming error on');
      // update progress form
      ProgressForm.StatusLabel.Caption := 'Verifying EEPROM';
      ProgressForm.StatusLabel.Repaint;
      // receive eeprom verify pass/fail
      if RBit(True, 2500) = 1 then CommError('EEPROM verify error on');
    end;
  end;
  CloseComm;
  ProgressForm.Hide;
end;

// Get hardware version
procedure GetHardwareVersion;
begin
  VersionMode := True;
  FindAnyHardware;
  ProgressForm.Hide;
  MessageDlg(('Hardware version ' + IntToStr(Version) + ' found on ' + CommString + '.'), mtInformation, [mbOK], 0);
end;

// Check for hardware on current comm port, then on com9..com1
procedure FindAnyHardware;
var
  i, FirstPort: integer;
begin
(*  // show progress form
  ProgressForm.Caption := 'Hardware';
  ProgressForm.StatusLabel.Caption := '';
  ProgressForm.Show;
   check com9..com1 for hardware
  FirstPort := CommPort;
  for i := 10 downto 1 do
  begin
    if i <> 10 then CommPort := i;
    if (i = 10) or (i <> FirstPort) then
    begin
      ProgressForm.StatusLabel.Caption := 'Checking ' + CommString;
      ProgressForm.StatusLabel.Repaint; *)
      if HardwareFound then Exit;
{    end;
  end;}
//  CommPort := 9;
  CommError('No hardware found on');
end;

// Check for hardware on current comm port
function HardwareFound: boolean;
var
  i: integer;
begin
  try
    // in case error, result is false
    Result := False;
    // disallow abort
    AbortMode := False;
    // check hardware
    OpenComm;
    // do reset and send connect string + blanks for echoes
    ResetHardware;
    TByte($F9);
    LFSR := Byte('P');
    for i := 1 to 250 do TByte(IterateLFSR or $FE);
    for i := 1 to 250 + 8 do TByte($F9);
    TComm;
    // receive connect string
    for i := 1 to 250 do if RBit(False, 100) <> IterateLFSR then HardwareLostError;
    // receive version
    for i := 1 to 8 do Version := Version shr 1 and $7F or RBit(False, 50) shl 7;
    // if find hardware mode, send shutdown command and reset hardware to reboot
    if VersionMode then
    begin
      TLong(0);
      TComm;
      ResetHardware;
      CloseComm;
    end;
    // allow abort
    AbortMode := True;
    // connected, result is true
    Result := True;
  except
    // error, result is false, allow abort
    on EAbort do AbortMode := True;
  end;
end;

// Reset hardware
procedure ResetHardware;
begin
  // generate reset pulse via dtr and wait 100ms
  SerSetDTR(CommHandle, True);
//  DelayMilliseconds
  Delay(25);
  SerSetDTR(CommHandle, False);
  Delay(90);
end;

/////////////////////
//  Comm Routines  //
/////////////////////

// Open comm port
procedure OpenComm;
begin
  CommOpen := False;
  CommHandle := SerOpen(CommString);
  if CommHandle = -1 then CommError('Unable to open');
  SerSetParams(CommHandle, 115200, 8, NoneParity, 1, []);
  TxBuffLength := 0;
  RxBuffStart := 0;
  RxBuffEnd := 0;
  CommOpen := True;
end;

// Add encoded long (11 bytes) to comm buffer
procedure TLong(x: cardinal);
var
  i: integer;
begin
  for i := 0 to 10 do
  begin
    TByte($92 or -Ord(i=10) and $60 or x and 1 or x and 2 shl 2 or x and 4 shl 4);
    x := x shr 3;
  end;
end;

// Add byte to comm buffer
procedure TByte(x: byte);
begin
  TxBuff[TxBuffLength] := x;
  Inc(TxBuffLength);
  if TxBuffLength = TxBuffSize then TComm;
end;

// Transmit comm buffer
procedure TComm;
var
  BytesWritten: Longint;
begin
  BytesWritten := SerWrite(CommHandle, TxBuff, TxBuffLength);
  if BytesWritten = -1 then
    CommError('Unable to write to');
  if BytesWritten <> TxBuffLength then
    CommError('Transmit stall on');
  TxBuffLength := 0;
end;

// Receive comm response via echoed byte
function RBit(Echo: boolean; Timeout: integer): byte;
var
  Ticks: cardinal;
  i: integer;
  
begin
  Ticks := GetTickCount;
  i:= 0;
  while GetTickCount - Ticks < Timeout do
  begin
inc(i);
//delay(50);
writeln(i, ' <> ', Ticks,' = ',GetTickCount);
//readln;

    if Echo then
    begin
      TByte($F9);
      TComm;
      DelayMilliseconds(25);
    end;
    if RxBuffStart = RxBuffEnd then RComm;
    if RxBuffStart <> RxBuffEnd then
    begin
      Result := RxBuff[RxBuffStart] - $FE;
      Inc(RxBuffStart);
      if Result and $FE = 0 then Exit;
      HardwareLostError;
    end;
  end;
  HardwareLostError;
end;

// Receive any data into comm buffer
procedure RComm;
var BytesRead: Longint;
begin
BytesRead := 0;
  RxBuffStart := 0;
  RxBuffEnd := 0;
  BytesRead := SerRead(CommHandle, RxBuff, RxBuffSize);
  if BytesRead = -1 then
    CommError('Unable to read from');

  RxBuffEnd := DWord(BytesRead);
  writeln(RxBuffEnd, ' = ', RxBuff[0]);
end;

// Hardware lost error
procedure HardwareLostError;
begin
  CommError('Hardware lost on');
end;

// Comm error
procedure CommError(Msg: string);
begin
  if CommOpen then CloseComm;
  if AbortMode then
  begin
    ProgressForm.Hide;
    MessageDlg(Msg + ' ' + CommString + '.', mtError, [mbOK], 0);
  end;
  Abort;
end;

// Close comm port
procedure CloseComm;
begin
  SerClose(CommHandle);
  CommOpen := False;
end;

// Return comm port string
function CommString: string;
begin
{$IFDEF Windows}
  Result := 'COM' + IntToStr(CommPort);
{$ENDIF}

{$IFDEF Unix}
  Result := '/dev/ttyUSB' + IntToStr(CommPort);
{$ENDIF}
end;


/////////////////////
//  Miscellaneous  //
/////////////////////

// Delay milliseconds
procedure DelayMilliseconds(ms: cardinal);
var
  Ticks: cardinal;
begin
Delay(ms);
{  Ticks := GetTickCount;
  while GetTickCount - Ticks < ms do;}
end;

// Iterate LFSR
function IterateLFSR: byte;
begin
  Result := LFSR and $01;
  LFSR := LFSR shl 1 and $FE or (LFSR shr 7 xor LFSR shr 5 xor LFSR shr 4 xor LFSR shr 1) and 1;
end;

{ $IFDEF Unix}
function GetTickCount: TClock;
var
  t : tms;
begin
Result := fpTimes(t);
end;
{ $ENDIF}

end.

